#!/usr/bin/perl

use strict;
use Getopt::Long qw(:config no_ignore_case);

my ($mon_ips, $key);
my $CEPH = '/usr/bin/ceph';
my $HELP = 0;

my %STATUSCODE = ( 
	'OK'       => '0',
	'WARNING'  => '1',
	'CRITICAL' => '2',
	'UNKNOWN'  => '3',
);

my $usage = <<EOF;
 
This plugin checks the health of a ceph cluster.

Usage: check_ceph

Really that's it!

EOF

if (! -x $CEPH) {
	print "Where is the ceph binary?  Not here: $CEPH\n";
	exit ($STATUSCODE{'UNKNOWN'});

}

# run ceph command and grab output
my $cephcmd = "ceph health";
open(CEPHOUT, "$cephcmd |") or exit ($STATUSCODE{'UNKNOWN'});
while (defined( my $line = <CEPHOUT> )) {
	chomp $line;

	my $health_status = '';
	# matching on the expected second line of output with the arrow pointing to the right (->)
	if (($health_status) = ($line =~ /.+\ ->\ \'(HEALTH_(OK|WARN|ERR).*)\'.*/)) {
		print "status: '$health_status'\n";
		if ($health_status =~ /^HEALTH_OK/) {
			exit ($STATUSCODE{'OK'});
		}
		elsif ($health_status =~ /^HEALTH_WARN/) {
			exit ($STATUSCODE{'WARNING'});
		}
		elsif ($health_status =~ /^HEALTH_ERR/) {
			exit ($STATUSCODE{'CRITICAL'});
		}
		else {
			exit ($STATUSCODE{'UNKNOWN'});
		}
	}
}
close (CEPHOUT);

# if we got to here, something didn't work right...
exit ($STATUSCODE{'UNKNOWN'});


# output format is...
# root@ladder0:~# ceph health
# 2011-02-11 11:30:11.299622 mon <- [health]
# 2011-02-11 11:30:11.300001 mon0 -> 'HEALTH_OK' (0)
# 
# fatty:src 11:30 AM $ ./ceph health
# 2011-02-11 11:30:06.810784 mon <- [health]
# 2011-02-11 11:30:06.842182 mon1 -> 'HEALTH_WARN Some PGs are: degraded' (0)
# 11:32
# it's always HEALTH_OK, HEALTH_WARN or HEALTH_ERR and some random string


