#!/usr/bin/perl
# check to make sure machine is running in full duplex

use strict;

my @interfaces = ();

open (my $fh, '<', '/etc/udev/rules.d/70-persistent-net.rules');

if (!$fh) {
	print "Unable to load /etc/udev/rules.d/70-persistent-net.rules!\n";
	exit 3;
}

while (<$fh>) {
	next if $_ =~ m/^(?:#|(?:\s+?)$)/; #skipping over blank lines and comments
	$_ =~ m/NAME="(.*)"/;
	push(@interfaces, $1);
}

for my $interface (@interfaces) {
	if ( -d "/sys/class/net/$interface" ) {
		my $duplex = `ethtool $interface | grep Duplex | awk '{ print \$2 }'`;
		chomp $duplex;
		if ( $duplex ne 'Full' ) {
			print "Bad duplex setting on $interface: $duplex\n";
			exit 2;
		}
	} else {
		print "Bad duplex setting on $interface\n";
		exit 2;
	}
        if ( -d "/sys/class/net/$interface" ) {
                my $speed = `ethtool $interface | grep Speed | awk '{ print \$2 }'`;
                chomp $speed;
                if ( $speed ne '1000Mb/s') {
                        print "Link speed incorrect on $interface: $speed\n";
                        exit 2;
                }
        }
}

print "Duplex is good\n";
exit 0;
