#!/bin/bash
#
# simple script to check that all bound things are returning data
#

export PATH="/sbin:$PATH";

haproxyconfig="/etc/haproxy/haproxy.cfg";
testuser="mailtest";
testpass="aSdF9630";
tmpfile=/tmp/check_haproxy.$$;
vipdev=eth1;


touch $tmpfile;
chmod 600 $tmpfile;

#make sure the process is actually running!
isrunning=`killall -0 haproxy`
if [[ $isrunning -ne 0 ]]; then
	echo -ne "Critical: HAProxy is not running!\n";
	exit 2;
fi;

# sanity check
iptest=`ip addr show $vipdev | grep "inet " | wc -l`
if [ $iptest -lt 2 ]; then
	echo -ne "OK: not master\n";
	exit 0;
fi;


FAILED="";
for ipport in `cat $haproxyconfig | grep bind | awk '{print $2}'`;
do
	ip=`echo "$ipport" | cut -d: -f1`;
	port=`echo "$ipport" | cut -d: -f2`;

	# test for master
	testaddr=`ip addr show $vipdev | grep "$ip/32"`;
	if [ -z "$testaddr" ]; then
		if [ -z "$FAILED" ]; then
			FAILED="$ipport (missing)";
		else
			FAILED="$FAILED $ipport (missing)";
		fi;
		continue;
	fi;

	# test services
	COMMAND="";
	TESTMSG="";
	if [ $port -eq 110 -o $port -eq 143 -o $port -eq 25 -o $port -eq 587 -o $port -eq 10024 -o $port -eq 10029 ]; then
		COMMAND="nc -w 5 $ip $port";
	elif [ $port -eq 995 -o $port -eq 993 -o $port -eq 465 ]; then
		COMMAND="stunnel -D3 -c -f -r $ipport";
	fi;

	if [ ! -z "$COMMAND" ]; then
		# pop/pops
		if [ $port -eq 110 -o $port -eq 995 ]; then
			(
				sleep 1;
				echo -ne "USER $testuser\n";
				sleep 1;
				echo -ne "PASS $testpass\n";
				sleep 3;
			) | $COMMAND > $tmpfile 2>&1;
			TESTMSG=`tail -1 $tmpfile | grep "OK"`;

		# imap/imaps
		elif [ $port -eq 143 -o $port -eq 993 ]; then
			(
				sleep 1;
				echo -ne ". LOGIN $testuser $testpass\n";
				sleep 3;
			) | $COMMAND > $tmpfile 2>&1;
			TESTMSG=`tail -1 $tmpfile | grep "OK"`;

		# smtp/smtps/submission
		elif [ $port -eq 25 -o $port -eq 465 -o $port -eq 587 -o $port -eq 10029 ]; then
			encpass=`perl -MMIME::Base64 -e "print encode_base64(\"\\000$testuser\\000$testpass\")"`;
			(
				sleep 1;
				echo -ne "EHLO `hostname`\n";
				sleep 1;
				echo -ne "AUTH PLAIN $encpass\n";
				sleep 1;
			) | $COMMAND > $tmpfile 2>&1;
			TESTMSG=`tail -1 $tmpfile | grep "Authentication successful\|authentication not enabled"`;

		# amavis
		elif [ $port -eq 10024 ]; then
			echo -ne "QUIT\n" | $COMMAND > $tmpfile 2>&1;
			TESTMSG=`grep "service ready" $tmpfile`;

		fi;
	fi;

	if [ -z "$TESTMSG" ]; then
		if [ -z "$FAILED" ]; then
			FAILED="$ipport";
		else
			FAILED="$FAILED $ipport";
		fi;
	fi;
done;

rm -f $tmpfile;

if [ -z "$FAILED" ]; then
	echo -ne "OK - all services up.\n";
	exit 0;
else
	echo -ne "FAILED services: $FAILED\n";
	exit 2;
fi;
