#!/bin/sh
# Runs 'chassis status' ipmi command to verify ipmi works.
# Takes basically the same flags as ipmitool.
# -I (ipmi interface)
# -H (host)
# -U (user)
# -P (password)

address=$1

set -- `getopt I:H:U:P: "$@"`

interface=lanplus
user=root

while [ $# -gt 0 ] ; do
  case "$1" in
    -I) interface=$2 ; shift ;;
    -H) host=$2; shift ;;
    -U) user=$2; shift ;;
    -P) password=$2; shift ;;
    --) shift; break;
  esac

  shift
done

if [ -z "$interface" ] ; then
  echo "No value for IPMI interface (-I)."
  exit 3
fi

if [ -z "$user" ] ; then
  echo "No value for user (-U). You must specify this."
  exit 3
fi

if [ -z "$host" ] ; then
  echo "No value for host (-H). You must specify this."
  exit 3
fi

if [ -z "$password" ] ; then
  echo "No value for password (-P). You must specify this."
  exit 3
fi

ipmitool -I $interface -H $host -U $user -P $password chassis status
if [ $? -ne 0 ] ; then
  echo "IPMI failed"
  exit 2
fi

echo "IPMI success"
exit 0
