#!/bin/bash
#
# check_varnish.sh
#
# A nagios plugin to check the status of a varnish web cache using varnishadm.
#
# Example: ./check_varnish.sh -i 127.0.0.1 -p 6082
#
#################################################################################

PATH=/sbin:/bin:/usr/sbin:/usr/bin

E_SUCCESS="0"
E_WARNING="1"
E_CRITICAL="2"
E_UNKNOWN="3"

PROGNAME=`/usr/bin/basename $0`
CHECK_VARNISHD=`/bin/ps ax | /bin/grep varnishd | /bin/grep -v grep`

print_help() {
        echo ""
        echo "This plugin checks the status of a varnish web cache using varnishadm"
        echo ""
        echo "Usage: $PROGNAME -i <ip address>  -p <port>"
        echo ""
}

if [ $# -lt 4 ]; then
        print_help
        exit $E_UNKNOWN
fi

if [ -z "$CHECK_VARNISHD" ]; then
	echo "CRITICAL: varnishd is not running, unable to check status"
	exit $E_CRITICAL
fi

while test -n "$1"; do
	case "$1" in
		--help | -h)
			print_help
			exit $E_UNKNOWN
			;;
		-i)
			IP=$2
			shift
			;;
		-p)	
			PORT=$2
			shift
			;;
	esac
	shift
done

COMMAND=`/usr/bin/varnishadm -T $IP\:$PORT -S /etc/varnish/secret status  2>&1 | /usr/bin/head -1`

if [ "$COMMAND" = "Child in state running" ]; then
    echo "VARNISH OK: $COMMAND"
    exit ${E_SUCCESS}

else
    echo "VARNISH CRITICAL: $COMMAND"
    exit ${E_CRITICAL}
fi
